// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "sdrehen.hpp"
#include <qpushbutton.h>
#include <qwmatrix.h>
#include <qlayout.h>

SDrehen::SDrehen(QWidget *parent , QImage *datenAddr , QImage *backupAddr) : QDialog(parent , "drehen" , true) {
  daten = datenAddr;
  backup = backupAddr;
  QGridLayout *container = new QGridLayout(this , 4 , 3 , 10 , -1 , "container");
  QPushButton *spin90 = new QPushButton("&90" , this , "spin90");
  QPushButton *spin180 = new QPushButton("&180" , this , "spin180");
  QPushButton *spin270 = new QPushButton("&270" , this , "spin270");
  rotator = new QSlider(1 , 719 , 1 , 1 , Qt::Horizontal , this , "rotator");
  info = new QLabel(this , "info");
  QPushButton *ok = new QPushButton(tr("&Finished") , this , "ok");
  info->setAlignment(Qt::AlignCenter);
  ok->setDefault(true);
  connect(spin90 , SIGNAL(clicked()) , this , SLOT(set90()));
  connect(spin180 , SIGNAL(clicked()) , this , SLOT(set180()));
  connect(spin270 , SIGNAL(clicked()) , this , SLOT(set270()));
  connect(rotator , SIGNAL(valueChanged(int)) , this , SLOT(machDVS(int)));
  connect(ok , SIGNAL(clicked()) , this , SLOT(accept()));
  container->addWidget(spin90 , 0 , 0);
  container->addWidget(spin180 , 0 , 1);
  container->addWidget(spin270 , 0 , 2);
  container->addMultiCellWidget(rotator , 1 , 1 , 0 , 2);
  container->addMultiCellWidget(info , 2 , 2 , 0 , 2);
  container->addMultiCellWidget(ok , 3 , 3 , 0 , 2);
  setCaption(tr("Rotate"));
  machDVS(1);
}

void SDrehen::machDVS(int wert) {
  double winkel = wert / 2.0;
  QWMatrix matrize;
  matrize.rotate(winkel);
  *daten = backup->xForm(matrize);
  info->setText(QString::number(winkel));
  emit modified();
}

