// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "sfshanp.hpp"
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlabel.h>

SFSHAnp::SFSHAnp(QWidget *parent , QImage *datenAddr , QImage *backupAddr) : QDialog(parent , "FSHAnp" , true) {
  daten = datenAddr;
  backup = backupAddr;
  QGridLayout *container = new QGridLayout(this , 4 , 3 , 10 , -1 , "container");
  hueAdj = new QSlider(-359 , 359 , 1 , 0 , Qt::Horizontal , this , "hueAdj");
  satAdj = new QSlider(-255 , 255 , 1 , 0 , Qt::Horizontal , this , "satAdj");
  valAdj = new QSlider(-255 , 255 , 1 , 0 , Qt::Horizontal , this , "valAdj");
  QPushButton *farbtNuller = new QPushButton(tr("&Start value") , this , "farbtNuller");
  QPushButton *saettNuller = new QPushButton(tr("&Start value") , this , "saettNuller");
  QPushButton *hellNuller = new QPushButton(tr("&Start value") , this , "hellNuller");
  connect(hueAdj , SIGNAL(valueChanged(int)) , this , SLOT(anpassen()));
  connect(satAdj , SIGNAL(valueChanged(int)) , this , SLOT(anpassen()));
  connect(valAdj , SIGNAL(valueChanged(int)) , this , SLOT(anpassen()));
  connect(farbtNuller , SIGNAL(clicked()) , this , SLOT(nullFarbt()));
  connect(saettNuller , SIGNAL(clicked()) , this , SLOT(nullSaett()));
  connect(hellNuller , SIGNAL(clicked()) , this , SLOT(nullHell()));
  container->addWidget(new QLabel(hueAdj , tr("&Hue:") , this , "descrH") , 0 , 0);
  container->addWidget(new QLabel(satAdj , tr("&Saturation:") , this , "descrS") , 1 , 0);
  container->addWidget(new QLabel(valAdj , tr("&Value:") , this , "descrV") , 2 , 0);
  container->addWidget(hueAdj , 0 , 1);
  container->addWidget(satAdj , 1 , 1);
  container->addWidget(valAdj , 2 , 1);
  container->addWidget(farbtNuller , 0 , 2);
  container->addWidget(saettNuller , 1 , 2);
  container->addWidget(hellNuller , 2 , 2);
  QPushButton *ok = new QPushButton(tr("&Finished") , this , "ok");
  ok->setDefault(true);
  connect(ok , SIGNAL(clicked()) , this , SLOT(accept()));
  container->addMultiCellWidget(ok , 3 , 3 , 0 , 2);
  setCaption(tr("HSV adjust"));
}

void SFSHAnp::anpassen() {
  int f , s , h;
  QColor farbe;
  if (backup->depth() > 8) //Bild benutzt keine Colortable?
      for (int x = 0 ; x < backup->width() ; x++)
	for (int y = 0 ; y < backup->height() ; y++) {
	  farbe.setRgb(backup->pixel(x , y));
	  farbe.hsv(&f , &s , &h);
	  farbe.setHsv(QMAX(QMIN(f + hueAdj->value() , 359) , 0) , QMAX(QMIN(s + satAdj->value() , 255) , 0) , QMAX(QMIN(h + valAdj->value(), 255) , 0));
	  daten->setPixel(x , y , farbe.rgb());
	}
    else	//sonst (Bild benutzt Colortable)
      for (int index = 0 ; index < backup->numColors() ; index++) {
	farbe.setRgb(backup->color(index));
	farbe.hsv(&f , &s , &h);
	farbe.setHsv(QMAX(QMIN(f + hueAdj->value() , 359) , 0) , QMAX(QMIN(s + satAdj->value() , 255) , 0) , QMAX(QMIN(h + valAdj->value(), 255) , 0));
	daten->setColor(index , farbe.rgb());
      }
  emit modified();
}

