// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "sgrad.hpp"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>

SGrad::SGrad(QWidget *parent , QImage *dataAdr , QImage *backupAdr) : QDialog(parent , "grad" , true) {
  data = dataAdr;
  backup = backupAdr;
  QGridLayout *container = new QGridLayout(this , 2 , 7 , 10 , -1 , "container");
  kurve = new SKurve(this);
  connect(kurve , SIGNAL(dataChanged()) , this , SLOT(createPreview()));
  container->addMultiCellWidget(kurve , 0 , 6 , 0 , 0);
  QLabel* info = new QLabel(this , "info");
  info->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
  connect(kurve , SIGNAL(positionChanged(const QString&)) , info , SLOT(setText(const QString&)));
  container->addWidget(info , 0 , 1);
  QPushButton* resetter = new QPushButton(tr("Re&set") , this , "resetter");
  connect(resetter , SIGNAL(clicked()) , this , SLOT(reset()));
  container->addWidget(resetter , 1 , 1);
  container->addWidget(new QLabel(tr("Use with:") , this , "descrUseWith") , 2 , 1);
  red = new QCheckBox(tr("&Red") , this , "red");
  green = new QCheckBox(tr("&Green") , this , "green");
  blue = new QCheckBox(tr("&Blue") , this , "blue");
  red->setChecked(true);
  green->setChecked(true);
  blue->setChecked(true);
  connect(red , SIGNAL(clicked()) , this , SLOT(createPreview()));
  connect(green , SIGNAL(clicked()) , this , SLOT(createPreview()));
  connect(blue , SIGNAL(clicked()) , this , SLOT(createPreview()));
  container->addWidget(red , 3 , 1);
  container->addWidget(green , 4 , 1);
  container->addWidget(blue , 5 , 1);
  QPushButton* ok = new QPushButton(tr("&Finished") , this , "ok");
  ok->setDefault(true);
  connect(ok , SIGNAL(clicked()), this , SLOT(accept()));
  container->addWidget(ok , 6 , 1);
  setCaption(tr("Gradation curve"));
}

void SGrad::createPreview() {
  QRgb alteFarbe;
  uchar NR , NG , NB;
  if (backup->depth() > 8)	//Bild benutzt keine Colortable?
      for (int x = 0 ; x < backup->width() ; x++)
	for (int y = 0 ; y < backup->height() ; y++) {
	  alteFarbe = backup->pixel(x , y);
	  NR = qRed(alteFarbe);
	  if (red->isChecked())
	    NR = kurve->data[NR];
	  NG = qGreen(alteFarbe);
	  if (green->isChecked())
	    NG = kurve->data[NG];
	  NB = qBlue(alteFarbe);
	  if (blue->isChecked())
	    NB = kurve->data[NB];
	  data->setPixel(x , y , qRgb(NR , NG , NB));
	}
    else
      for (int index = 0 ; index < backup->numColors() ; index++) {
	alteFarbe = backup->color(index);
	NR = qRed(alteFarbe);
	if (red->isChecked())
	  NR = kurve->data[NR];
	NG = qGreen(alteFarbe);
	if (green->isChecked())
	  NG = kurve->data[NG];
	NB = qBlue(alteFarbe);
	if (blue->isChecked())
	  NB = kurve->data[NB];
	data->setColor(index , qRgb(NR , NG , NB));
      }
  emit modified();
}

void SGrad::reset() {
  for (uchar zv = 1 ; zv > 0 ; zv++)
    kurve->data[zv] = zv;
  kurve->update();
  *data = backup->copy();
  emit modified();
}
