// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "spicview.hpp"
#include <qmessagebox.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qsettings.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qinputdialog.h>
#include <fstream>
#include <iomanip>
#include "extdlg/skompvs.hpp"
#include "extdlg/sdrehen.hpp"
#include "extdlg/srgbanp.hpp"
#include "extdlg/stiefenkonv.hpp"
#include "extdlg/sfshanp.hpp"
#include "extdlg/sgrad.hpp"
#include "extdlg/ssize.hpp"
#include "extdlg/sins.hpp"
#include "extdlg/slistedit.hpp"

const char *MODECHARS[] = {"M" , "M-" , "MW" , "MH" , "R" , "C"};

SPicView::SPicView(QWidget *parent) : QScrollView(parent , "pv" , WDestructiveClose) {
  display = false;
  recordingMacro = false;
  inhalt = new SPic(viewport());
  addChild(inhalt);
  connect(inhalt , SIGNAL(markChanged4P(const QString &)) , this , SIGNAL(markChanged(const QString &)));
  connect(inhalt , SIGNAL(MPIChanged4P(const QString &)) , this , SIGNAL(MPIChanged(const QString &)));
  actFormat = (QString) 0;
  zoomMode = SZoomMode(QSettings().readNumEntry("/Yoltia/zoomModeInt"));
  undoList.setAutoDelete(true);
  redoList.setAutoDelete(true);
  setFocusPolicy(StrongFocus);	//for Hotkeys
  setHScrollBarMode(AlwaysOn);
  setVScrollBarMode(AlwaysOn);
  const char *icon16[] = {"16 16 2 1" , ". c None" , "a c #ff0000" , "...aaaaaaaaaa..." , "....aaaaaaaa...." , ".....aaaaaa....." , "a.....aaaa.....a" , "aa.....aa.....aa" , "aaa..........aaa" , "aaaa........aaaa" , "aaaaa......aaaaa" , "aaaaaa.....aaaaa" , "aaaaaaa.....aaaa" , "aaaaaaaa.....aaa" , "aaaaaaaaa.....aa" , "aaaaaaaaaa.....a" , "aaaaaaaaaaa....." , "aaaaaaaaaaaa...." , "aaaaaaaaaaaaa..."};
  setIcon(QPixmap(icon16));
}

void SPicView::viewportResizeEvent(QResizeEvent *) {
  zoom();
}

bool SPicView::createWithDir(const QString &dateipfad) {
  unsaved = false;
  listMan.setDirectoryMode(dateipfad);
  return load(dateipfad);
}

bool SPicView::createWithOwn(const QStringList ownList) {
  unsaved = false;
  listMan.setOwnMode(ownList);
  return load(listMan.act());
}

void SPicView::createWithOne(const int width , const int height , const int depth , const QRgb fillColor) {
  listMan.setOneMode();
  inhalt->bilddaten.create(width , height , 32);
  inhalt->bilddaten.fill(fillColor);
  inhalt->bilddaten = inhalt->bilddaten.convertDepth(depth , Qt::ThresholdDither);
  setCaption(tr("Not saved"));
  unsaved = true;
}

bool SPicView::createClipWithOne() {
  if (QApplication::clipboard()->image(QClipboard::Clipboard).isNull())
    return false;
  listMan.setOneMode();
  inhalt->bilddaten = QApplication::clipboard()->image(QClipboard::Clipboard).copy();
  setCaption(tr("Not saved"));
  unsaved = true;
  return true;
}

bool SPicView::load(const QString &dateipfad) {
  QSettings einst;
  QStringList letzte(einst.readListEntry("/Yoltia/history/list"));
  if (letzte.find(dateipfad) != letzte.end())	//Bereits Eintrag in den letzten Dateien?
    letzte.remove(letzte.find(dateipfad));	//Eintrag entfernen
  bool ergebnis = inhalt->bilddaten.load(dateipfad);
  if (ergebnis) {	//Bild loadn erfolgreich?
    letzte.prepend(dateipfad);	//Neuen Eintrag voranstellen
    inhalt->killMark();
    undoList.clear();
    redoList.clear();
    unsaved = false;
    zoomMode = SZoomMode(einst.readNumEntry("/Yoltia/zoomModeInt"));
    zoom();
    actFormat = QString(QImageIO::imageFormat(dateipfad)).remove("RAW");
    refreshStatus();
    setCaption(dateipfad);
    einst.writeEntry("/Yoltia/history/dir" , dateipfad.left(dateipfad.findRev("/" , -2)));
  }
  if ((int) letzte.count() > einst.readNumEntry("/Yoltia/history/count" , 10))	//Maximale Listenl?nge erreicht?
    letzte.erase(letzte.at(einst.readNumEntry("/Yoltia/history/count" , 10)) , --letzte.end());	//L?schen der letzten Eintr?ge
  einst.writeEntry("/Yoltia/history/list" , letzte);
  return ergebnis;
}

void SPicView::zoom() {
  if (isVisible()) {	//sonst kommt es zu Fehlern bei der Erstellung des Pixmaps, da wenn nicht sichtbar visible*() == 0
    QSettings set;
    const int useableHeight = visibleHeight() - set.readNumEntry("/Yoltia/markspace" , 15) , useableWidth = visibleWidth() - set.readNumEntry("/Yoltia/markspace" , 15);
    QImage scaled;	//Zuerst zwischenspeichern der gezoomten Version zur Pr?fung, ob ?berhaupt Anzeige m?glich
    switch (zoomMode) {
      case maxA :
	if ((double) inhalt->bilddaten.height() / (double) inhalt->bilddaten.width() > (double) useableHeight / (double) useableWidth) {	//nach H?he skalieren?
	    scaled = inhalt->bilddaten.scaleHeight(useableHeight);
	    inhalt->zoom = (double) useableHeight / (double) inhalt->bilddaten.height();
	  } else {	//sonst nach Breite skalieren
	    scaled = inhalt->bilddaten.scaleWidth(useableWidth);
	    inhalt->zoom = (double) useableWidth / (double) inhalt->bilddaten.width();
	}
	break;
      case maxB :
	scaled = inhalt->bilddaten.scaleWidth(useableWidth);
	inhalt->zoom = (double) useableWidth / (double) inhalt->bilddaten.width();
	break;
      case maxH :
	scaled = inhalt->bilddaten.scaleHeight(useableHeight);
	inhalt->zoom = (double) useableHeight / (double) inhalt->bilddaten.height();
	break;
      case full :
	scaled = inhalt->bilddaten;
	inhalt->zoom = 1.0;
	break;
      case maxAFZG :
	if (inhalt->bilddaten.width() > useableWidth or inhalt->bilddaten.height() > useableHeight)	//Irgendwie zu gro??
	    if ((double) inhalt->bilddaten.height() / (double) inhalt->bilddaten.width() > (double) useableHeight / (double) useableWidth) {	//nach H?he skalieren?
		scaled = inhalt->bilddaten.scaleHeight(useableHeight);
		inhalt->zoom = (double) useableHeight / (double) inhalt->bilddaten.height();
	      } else {	//sonst nach Breite skalieren
		scaled = inhalt->bilddaten.scaleWidth(useableWidth);
		inhalt->zoom = (double) useableWidth / (double) inhalt->bilddaten.width();
	    }
	  else {	//sonst gar nicht skalieren
	    scaled = inhalt->bilddaten;
	    inhalt->zoom = 1.0;
	}
	break;
      case custom :
	scaled = inhalt->bilddaten.scaleHeight(qRound(inhalt->bilddaten.height() * inhalt->zoom));
    }
    display = !scaled.isNull() and scaled.width() < 4001 and scaled.height() < 4001;
    if (display) {
	emit zoomChanged(QString("%1: %2 %").arg(MODECHARS[zoomMode]).arg(qRound(inhalt->zoom * 100.0)));
	inhalt->drawdaten.convertFromImage(scaled , Qt::ColorOnly);
	if (inhalt->drawdaten.height() > useableHeight and inhalt->drawdaten.width() > useableWidth)	//Bild beidseitig zu gro??
	    inhalt->resize(inhalt->drawdaten.width() + 20 , inhalt->drawdaten.height() + 20);	//Inhaltsgr??e = Bildgr??e
	  else
	    if (inhalt->drawdaten.width() > useableWidth)	//Bild zu breit?
		inhalt->resize(inhalt->drawdaten.width() + 20 , visibleHeight());	//Inhaltsbreite = Bildbreite, Inhaltsh?he = sichtbare H?he
	      else
		if (inhalt->drawdaten.height() > useableHeight)	//Bild zu hoch?
		    inhalt->resize(visibleWidth() , inhalt->drawdaten.height() + 20);	//Inhaltsh?he = Bildh?he, Inhaltsbreite = sichtbare Breite
		  else
		  inhalt->resize(visibleWidth() , visibleHeight());	//sonst Inhaltsgr??e = sichtbare Gr??e
      } else {
	emit zoomChanged(0);
	inhalt->drawdaten = (QPixmap) 0;
	inhalt->resize(visibleWidth() , visibleHeight());
    }
    inhalt->update();
  }
}

void SPicView::command(int spec) {
  if (recordingMacro and spec < 200 and spec > 99)	//Record macro?
    macro.append(spec);
  switch (spec) {
    //Control Hotkeys START
    case 0 :	//Vorhergehendes Bild im Verzeichnis loadn
      if (listMan.isBrowseable() and notifyChanges())
	load(listMan.prev());
      break;
    case 1 :	//Nachfolgendes Bild im Verzeichnis loadn
      if (listMan.isBrowseable() and notifyChanges())
	load(listMan.next());
      break;
    case 2 :	//Erstes Bild im Verzeichnis loadn
      if (listMan.isBrowseable() and notifyChanges())
	load(listMan.first());
      break;
    case 3 :	//Letztes Bild im Verzeichnis loadn
      if (listMan.isBrowseable() and notifyChanges())
	load(listMan.last());
      break;
    case 4 :	//Bild unter aktuellem Dateipfad mit aktuellem Format speichern
        if (unsaved and !QImage::outputFormatList().grep(actFormat).isEmpty() and listMan.act()) {	//?nderungen vorgenommen, aktuelles Format speicherbar und Dateiname bekannt?
          emit meldung(tr("Save: Writing \"%1\"...").arg(listMan.act()) , LONGDELAY);
          speichere(listMan.act() , actFormat);
        }
      break;
    case 5 :	//Undo
      if (undoList.count()) {
          inhalt->killMark();
          redoList.append(new QImage(inhalt->bilddaten.copy()));
          recoverBackup();
        } else
          emit meldung(tr("Undo: No backups found, aborted.") , SHORTDELAY);
      break;
    case 6 :	//Redo
      if (redoList.count()) {
          inhalt->killMark();
          backup();
          inhalt->bilddaten = redoList.last()->copy();
          redoList.remove();	//last == current
          refreshStatus();
          zoom();
        } else
          emit meldung(tr("Redo: No backups found, aborted.") , SHORTDELAY);
      break;
    case 7 :	//Kopieren
      if (inhalt->existsFixedMark()) {
          emit meldung(tr("Copy: Copying marked region...") , SHORTDELAY);
          QApplication::clipboard()->setImage(inhalt->bilddaten.copy(inhalt->markX , inhalt->markY , inhalt->markB , inhalt->markH) , QClipboard::Clipboard);
        } else {
          emit meldung(tr("Copy: Copying whole picture...") , SHORTDELAY);
          QApplication::clipboard()->setImage(inhalt->bilddaten , QClipboard::Clipboard);
      }
      break;
    case 8 :	//Reload
      if (listMan.act() and notifyChanges()) {
        emit meldung(tr("Reload: Loading file \"%1\"...").arg(listMan.act()) , LONGDELAY);
        load(listMan.act());
      }
      break;
    case 9 :	//Zoom-In (+20 %)
      zoomMode = custom;
      inhalt->zoom += 0.2;
      zoom();
      break;
    case 10 :	//Zoom-In (+1 %)
      zoomMode = custom;
      inhalt->zoom += 0.01;
      zoom();
      break;
    case 11 :	//Zoom-In (+50 %)
      zoomMode = custom;
      inhalt->zoom += 0.5;
      zoom();
      break;
    case 12 :	//Zoom-Out (-20 %)
      zoomMode = custom;
      inhalt->zoom -= 0.2;
      zoom();
      break;
    case 13 :	//Zoom-Out (-1 %)
      zoomMode = custom;
      inhalt->zoom -= 0.01;
      zoom();
      break;
    case 14 :	//Zoom-Out (-50 %)
      zoomMode = custom;
      inhalt->zoom -= 0.5;
      zoom();
      break;
    case 15 :	//Delete only file
      if (listMan.act())	//Bild ?berhaupt Datei zugeordnet?
        if (QMessageBox::information(this , tr("Yoltia - Delete file?") , tr("The current file will be unrecoverable deleted.\nDo you want to delete the file anyway?") , QMessageBox::Yes , QMessageBox::No , QMessageBox::NoButton) == QMessageBox::Yes) {
          if (QFile::remove(listMan.act()))	//Delete successful?
              emit meldung(tr("Delete file: Deleting file \"%1\"...").arg(listMan.act()) , LONGDELAY);
            else
              emit meldung(tr("Delete file: Deleting failed, aborted.") , SHORTDELAY);
          unsaved = true;
          listMan.setOneMode();
          actFormat = (QString) 0;
          refreshStatus();
        }
      break;
    case 16 :	//Delete file and picture
      if (listMan.act())
        if (QMessageBox::information(this , tr("Yoltia - Delete file?") , tr("The current file will be unrecoverable deleted.\nDo you want to delete the file anyway?") , QMessageBox::Yes , QMessageBox::No , QMessageBox::NoButton) == QMessageBox::Yes)
          if (QFile::remove(listMan.act())) {	//Delete successful?
              emit meldung(tr("Delete file: Deleting file \"%1\"...").arg(listMan.act()) , SHORTDELAY);
              load(listMan.atActPos());
            } else
              emit meldung(tr("Delete file: Deleting failed, aborted.") , SHORTDELAY);
      break;
    case 17 :	//Set zoom mode to maxA
      zoomMode = maxA;
      zoom();
      QSettings().writeEntry("/Yoltia/zoomModeInt" , 0);
      break;
    case 18 :	//Set zoom mode to maxAFZG
      zoomMode = maxAFZG;
      zoom();
      QSettings().writeEntry("/Yoltia/zoomModeInt" , 1);
      break;
    case 19 :	//Set zoom mode to maxB
      zoomMode = maxB;
      zoom();
      QSettings().writeEntry("/Yoltia/zoomModeInt" , 2);
      break;
    case 20 :	//Set zoom mode to maxH
      zoomMode = maxH;
      zoom();
      QSettings().writeEntry("/Yoltia/zoomModeInt" , 3);
      break;
    case 21 :	//Set zoom mode to full
      zoomMode = full;
      zoom();
      QSettings().writeEntry("/Yoltia/zoomModeInt" , 4);
      break;
    case 22 :	//Execute macro
      if (macro.empty())
	  emit meldung(tr("Macro is empty, could not be executed.") , SHORTDELAY);
	else {
	  recordingMacro = false;
	  for (QValueList<int>::iterator it = macro.begin() ; it != macro.end() ; it++)        //Forgive me this little recursion ;-)
	    command(*it);
      }
      break;
    case 23 :   //Goto
      {
        bool button;
        int gotoPos = QInputDialog::getInteger(tr("Goto") , tr("Enter the position of the picture:") , listMan.currentPos() + 1 , 1 , listMan.count() , 1 , &button , this , "GotoDlg") - 1;
        if (button)
          load(listMan.goTo(gotoPos));
      }
      break;
    //Image Editing Hotkeys START
    case 100 :	//Cut free marked region
      if (inhalt->existsFixedMark()) {
	  inhalt->killMark();
	  backup();
	  cleanLists();
	  inhalt->bilddaten = inhalt->bilddaten.copy(inhalt->markX , inhalt->markY , inhalt->markB , inhalt->markH);
	  unsaved = true;
	  refreshStatus();
	  zoom();
	} else
	  emit meldung(tr("Crop: No (fixed) marking found, aborted.") , SHORTDELAY);
      break;
    case 101 :	//Bild invertieren
      emit meldung(tr("Invert: Inverting...") , SHORTDELAY);
      inhalt->bilddaten.invertPixels();
      unsaved = true;
      refreshStatus();
      zoom();
      break;
    case 102 :	//Mirror vertically
      emit meldung(tr("Mirror vertically: Mirroring...") , SHORTDELAY);
      inhalt->bilddaten = inhalt->bilddaten.mirror();
      unsaved = true;
      refreshStatus();
      zoom();
      break;
    case 103 :	//Mirror horizontally
      emit meldung(tr("Mirror horizontally: Mirroring...") , SHORTDELAY);
      inhalt->bilddaten = inhalt->bilddaten.mirror(true , false);
      unsaved = true;
      refreshStatus();
      zoom();
      break;
    case 104 :	//Fit to max watched size (width AND height (depens on watch settigs) should be at most as big as upper limits)
      {
	QSettings set;
	bool watchHeight = set.readBoolEntry("/Yoltia/Watch/WatchHeightMax" , false) , watchWidth = set.readBoolEntry("/Yoltia/Watch/WatchWidthMax" , false);
	if (watchHeight or watchWidth) {	//Watching at least one
            enum SFit {scaleWidth , scaleHeight , scaleBoth , nothing};
            SFit todo = nothing;
            int height = set.readNumEntry("/Yoltia/Watch/heightMaxValue" , 100) , width = set.readNumEntry("/Yoltia/Watch/widthMaxValue" , 100);
            if (watchHeight and watchWidth and (inhalt->bilddaten.width() > width or inhalt->bilddaten.height() > height))
                todo = scaleBoth;
              else
                if (watchHeight and !watchWidth and inhalt->bilddaten.height() > height)
                    todo = scaleHeight;
                  else
                    if (!watchHeight and watchWidth and inhalt->bilddaten.width() > width)
                      todo = scaleWidth;
            if (todo != nothing) {
                inhalt->killMark();
                backup();
                cleanLists();
                switch (todo) {
                  case scaleWidth :
                    inhalt->bilddaten = inhalt->bilddaten.smoothScale(width , inhalt->bilddaten.height() , QImage::ScaleMin);
                    break;
                  case scaleHeight :
                    inhalt->bilddaten = inhalt->bilddaten.smoothScale(inhalt->bilddaten.width() , height , QImage::ScaleMin);
                    break;
                  default :     //case scaleBoth :
                    inhalt->bilddaten = inhalt->bilddaten.smoothScale(width , height , QImage::ScaleMin);
                }
                unsaved = true;
                refreshStatus();
                zoom();
              } else
                emit meldung(tr("Suit size: Image size does not violate upper limits, aborted.") , SHORTDELAY);
	  } else
	    emit meldung(tr("Suit size: Watching of upper limits deactivated, aborted.") , SHORTDELAY);
      }
      break;
    case 105 :	//Fit to min watched size (width AND height (depens on watch settigs) should be at least as big as lower limits)
      {
	QSettings set;
	bool watchHeight = set.readBoolEntry("/Yoltia/Watch/WatchHeightMin" , false) , watchWidth = set.readBoolEntry("/Yoltia/Watch/WatchWidthMin" , false);
        if (watchHeight or watchWidth) {
	    enum SFit {scaleWidth , scaleHeight , scaleBoth , nothing};
            SFit todo = nothing;
            int height = set.readNumEntry("/Yoltia/Watch/heightMinValue" , 100) , width = set.readNumEntry("/Yoltia/Watch/widthMinValue" , 100);
            if (watchHeight and watchWidth and (inhalt->bilddaten.width() < width or inhalt->bilddaten.height() < height))
                todo = scaleBoth;
              else
                if (watchHeight and !watchWidth and inhalt->bilddaten.height() < height)
                    todo = scaleHeight;
                  else
                    if (!watchHeight and watchWidth and inhalt->bilddaten.width() < width)
                      todo = scaleWidth;
            if (todo != nothing) {
                inhalt->killMark();
                backup();
                cleanLists();
                switch (todo) {
                  case scaleWidth :
                    inhalt->bilddaten = inhalt->bilddaten.smoothScale(width , inhalt->bilddaten.height() , QImage::ScaleMax);
                    break;
                  case scaleHeight :
                    inhalt->bilddaten = inhalt->bilddaten.smoothScale(inhalt->bilddaten.width() , height , QImage::ScaleMax);
                    break;
                  default :     //case scaleBoth :
                    inhalt->bilddaten = inhalt->bilddaten.smoothScale(width , height , QImage::ScaleMax);
                }
                unsaved = true;
                refreshStatus();
                zoom();
              } else
                emit meldung(tr("Suit size: Image size does not violate lower limits, aborted.") , SHORTDELAY);
	  } else
	    emit meldung(tr("Suit size: Watching of lower limits deactivated, aborted.") , SHORTDELAY);
      }
      break;
    case 106 :	//Remove 1 col of pixels (right edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Ctrl" , 1);
        if (inhalt->bilddaten.width() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() - lineCount , inhalt->bilddaten.height());
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 107 :	//Remove 1 col of pixels (left edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Ctrl" , 1);
        if (inhalt->bilddaten.width() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(lineCount , 0 , inhalt->bilddaten.width() - lineCount , inhalt->bilddaten.height());
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 108 :	//Remove 1 row of pixels (lower edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Ctrl" , 1);
        if (inhalt->bilddaten.height() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() , inhalt->bilddaten.height() - lineCount);
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 109 :	//Remove 1 row of pixels (upper edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Ctrl" , 1);
        if (inhalt->bilddaten.height() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(0 , lineCount , inhalt->bilddaten.width() , inhalt->bilddaten.height() - lineCount);
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 110 :	//Remove 20 cols of pixels (right edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Shift" , 20);
        if (inhalt->bilddaten.width() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() - lineCount , inhalt->bilddaten.height());
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 111 :	//Remove 20 cols of pixels (left edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Shift" , 20);
        if (inhalt->bilddaten.width() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(lineCount , 0 , inhalt->bilddaten.width() - lineCount , inhalt->bilddaten.height());
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 112 :	//Remove 20 rows of pixels (lower edge)
      {
        const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Shift" , 20);
        if (inhalt->bilddaten.height() > lineCount) {
	    inhalt->killMark();
	    backup();
	    cleanLists();
	    inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() , inhalt->bilddaten.height() - lineCount);
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else
	    emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 113 :	//Remove 20 rows of pixels (upper edge)
      {
	const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/Shift" , 20);
	if (inhalt->bilddaten.height() > lineCount) {
	  inhalt->killMark();
	  backup();
	  cleanLists();
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , lineCount , inhalt->bilddaten.width() , inhalt->bilddaten.height() - lineCount);
	  unsaved = true;
	  refreshStatus();
	  zoom();
	} else
	  emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 114 :	//Remove 5 cols of pixels (right edge)
      {
	const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/WO" , 5);
	if (inhalt->bilddaten.width() > lineCount) {
	  inhalt->killMark();
	  backup();
	  cleanLists();
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() - lineCount , inhalt->bilddaten.height());
	  unsaved = true;
	  refreshStatus();
	  zoom();
	} else
	  emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 115 :	//Remove 5 cols of pixels (left edge)
      {
	const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/WO" , 5);
	if (inhalt->bilddaten.width() > lineCount) {
	  inhalt->killMark();
	  backup();
	  cleanLists();
	  inhalt->bilddaten = inhalt->bilddaten.copy(lineCount , 0 , inhalt->bilddaten.width() - lineCount , inhalt->bilddaten.height());
	  unsaved = true;
	  refreshStatus();
	  zoom();
	} else
	  emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 116 :	//Remove 5 rows of pixels (lower edge)
      {
	const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/WO" , 5);
	if (inhalt->bilddaten.height() > lineCount) {
	  inhalt->killMark();
	  backup();
	  cleanLists();
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() , inhalt->bilddaten.height() - lineCount);
	  unsaved = true;
	  refreshStatus();
	  zoom();
	} else
	  emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 117 :	//Remove 5 rows of pixels (upper edge)
      {
	const int lineCount = QSettings().readNumEntry("/Yoltia/Edge/WO" , 5);
	if (inhalt->bilddaten.height() > lineCount) {
	  inhalt->killMark();
	  backup();
	  cleanLists();
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , lineCount , inhalt->bilddaten.width() , inhalt->bilddaten.height() - lineCount);
	  unsaved = true;
	  refreshStatus();
	  zoom();
	} else
	  emit meldung(tr("Shorten edges: Picture too small, aborted.") , SHORTDELAY);
      }
      break;
    case 118 :	//Auto remove single-colored margin aroung pic
      {
	QColor colorMin , colorMax;
	QRgb pixel;
	int count = -1;
	bool changedFlag = false;
	const int colTol = QSettings().readNumEntry("/Yoltia/colTol" , 20);
	backup();	//cleanlists() is called at the end
	do {
	  count++;
	  colorMin = QColor(inhalt->bilddaten.pixel(0 , count));
	  colorMax = colorMin;
	  for (int i = 1 ; i < inhalt->bilddaten.width() ; i++) {
	    pixel = inhalt->bilddaten.pixel(i , count);
	    colorMin.setRgb(QMIN(qRed(pixel) , colorMin.red()) , QMIN(qGreen(pixel) , colorMin.green()) , QMIN(qBlue(pixel) , colorMin.blue()));
	    colorMax.setRgb(QMAX(qRed(pixel) , colorMax.red()) , QMAX(qGreen(pixel) , colorMax.green()) , QMAX(qBlue(pixel) , colorMax.blue()));
	  }
	} while ((count + 1 < inhalt->bilddaten.height()) and ((colorMax.red() - colorMin.red()) <= colTol and (colorMax.green() - colorMin.green()) <= colTol and (colorMax.blue() - colorMin.blue()) <= colTol));
	if (count) {
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , count , inhalt->bilddaten.width() , inhalt->bilddaten.height() - count);
	  changedFlag = true;
	}
	count = -1;	//Left edge
	do {
	  count++;
	  colorMin = QColor(inhalt->bilddaten.pixel(count , 0));
	  colorMax = colorMin;
	  for (int i = 1 ; i < inhalt->bilddaten.height() ; i++) {
	    pixel = inhalt->bilddaten.pixel(count , i);
	    colorMin.setRgb(QMIN(qRed(pixel) , colorMin.red()) , QMIN(qGreen(pixel) , colorMin.green()) , QMIN(qBlue(pixel) , colorMin.blue()));
	    colorMax.setRgb(QMAX(qRed(pixel) , colorMax.red()) , QMAX(qGreen(pixel) , colorMax.green()) , QMAX(qBlue(pixel) , colorMax.blue()));
	  }
	} while (count + 1 < inhalt->bilddaten.width() and ((colorMax.red() - colorMin.red()) <= colTol and (colorMax.green() - colorMin.green()) <= colTol and (colorMax.blue() - colorMin.blue()) <= colTol));
	if (count) {
	  inhalt->bilddaten = inhalt->bilddaten.copy(count , 0 , inhalt->bilddaten.width() - count , inhalt->bilddaten.height());
	  changedFlag = true;
	}
	count = -1;	//Right edge
	do {
	  count++;
	  colorMin = QColor(inhalt->bilddaten.pixel(inhalt->bilddaten.width() - count - 1 , 0));
	  colorMax = colorMin;	
	  for (int i = 1 ; i < inhalt->bilddaten.height() ; i++) {
	    pixel = inhalt->bilddaten.pixel(inhalt->bilddaten.width() - count - 1 , i);
	    colorMin.setRgb(QMIN(qRed(pixel) , colorMin.red()) , QMIN(qGreen(pixel) , colorMin.green()) , QMIN(qBlue(pixel) , colorMin.blue()));
	    colorMax.setRgb(QMAX(qRed(pixel) , colorMax.red()) , QMAX(qGreen(pixel) , colorMax.green()) , QMAX(qBlue(pixel) , colorMax.blue()));
	  }
	} while (count + 1 < inhalt->bilddaten.width() and ((colorMax.red() - colorMin.red()) <= colTol and (colorMax.green() - colorMin.green()) <= colTol and (colorMax.blue() - colorMin.blue()) <= colTol));
	if (count) {
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() - count , inhalt->bilddaten.height());
	  changedFlag = true;
	}
	count = -1;	//Lower edge
	do {
	  count++;
	  colorMin = QColor(inhalt->bilddaten.pixel(0 , inhalt->bilddaten.height() - count - 1));
	  colorMax = colorMin;
	  for (int i = 1 ; i < inhalt->bilddaten.width() ; i++) {
	    pixel = inhalt->bilddaten.pixel(i , inhalt->bilddaten.height() - count - 1);
	    colorMin.setRgb(QMIN(qRed(pixel) , colorMin.red()) , QMIN(qGreen(pixel) , colorMin.green()) , QMIN(qBlue(pixel) , colorMin.blue()));
	    colorMax.setRgb(QMAX(qRed(pixel) , colorMax.red()) , QMAX(qGreen(pixel) , colorMax.green()) , QMAX(qBlue(pixel) , colorMax.blue()));
	  }
	} while (count + 1 < inhalt->bilddaten.height() and ((colorMax.red() - colorMin.red()) <= colTol and (colorMax.green() - colorMin.green()) <= colTol and (colorMax.blue() - colorMin.blue()) <= colTol));
	if (count) {
	  inhalt->bilddaten = inhalt->bilddaten.copy(0 , 0 , inhalt->bilddaten.width() , inhalt->bilddaten.height() - count);
	  changedFlag = true;
	}      
	if (changedFlag) {
	    cleanLists();
	    unsaved = true;
	    refreshStatus();
	    zoom();
	  } else {
	    emit meldung(tr("No single colored margin found, you may increase the color tolerancy via Settings dialog.") , SHORTDELAY);
	    undoList.remove();	//Removes the backup
	}
      }
      break;
    //Menu commands START
    case 200 :	//Compression preview
      backup();
      {
        SKompVS kompVS(this , &inhalt->bilddaten , undoList.last());
        connect(&kompVS , SIGNAL(modified()) , this , SLOT(zoom()));
        connect(&kompVS , SIGNAL(modified()) , this , SLOT(refreshStatus()));
        kompVS.exec();
      }
      recoverBackup();
      break;
    case 201 :	//List editor
      if (listMan.isOneMode())
          meldung(tr("In the current browsing-mode there is no Browsing-List, that could be edited.") , SHORTDELAY);
        else {
          if (listMan.isDirMode() and QMessageBox::information(this , tr("Yoltia - Convert Browsing-List?") , tr("The current Browsing-List could not be edited,\nbecause the Directory mode is in use.\nDo you want to switch to Userdefined mode,\nto be able to edit the Browsing-List?") , QMessageBox::Yes , QMessageBox::No , QMessageBox::NoButton) == QMessageBox::Yes) {
            listMan.convertToOwn();	      
            refreshStatus();
          }
          if (listMan.isOwnMode()) {
            SListEdit listedit(this , listMan.filelist());
            if (listedit.exec())
              createWithOwn(listedit.getList());
          }
      }
      break;
    case 202 :	//Insert
      if (!QApplication::clipboard()->image(QClipboard::Clipboard).isNull()) {
          backup();
          inhalt->killMark();
          bool unsavedBackup = unsaved;
          unsaved = true;
          SIns ins(this , &inhalt->bilddaten , undoList.last());
          zoom();	//BUGFIX Need do press any button in dialog to see insertiation (emit SIns::modified() w/o being connected)
          refreshStatus();
          connect(&ins , SIGNAL(modified()) , this , SLOT(zoom()));
          connect(&ins , SIGNAL(modified()) , this , SLOT(refreshStatus()));
          if (ins.exec() == QDialog::Accepted)
              cleanLists();
            else {
              unsaved = unsavedBackup;
              recoverBackup();
          }
        } else
          emit meldung(tr("Insert: No useable clipboard, aborted.") , SHORTDELAY);
      break;
    case 203 :	//Rotate
      backup();
      inhalt->killMark();
      {
        const bool unsavedBackup = unsaved;
        unsaved = true;
        SDrehen drehen(this , &inhalt->bilddaten , undoList.last());
        zoom();
        refreshStatus();	//BUGFIX Need do press any button in dialog to see first rotation (emit SDrehen::modified() w/o being connected)
        connect(&drehen , SIGNAL(modified()) , this , SLOT(zoom()));
        connect(&drehen , SIGNAL(modified()) , this , SLOT(refreshStatus()));
        if (drehen.exec() == QDialog::Accepted)
            cleanLists();
          else {
            unsaved = unsavedBackup;
            recoverBackup();
        }
      }
      break;
    case 204 :	//Resize
      inhalt->killMark();
      backup();
      {
        const bool unsavedBackup = unsaved;
        unsaved = true;
        SSize size(this , &inhalt->bilddaten , undoList.last());
        connect(&size , SIGNAL(modified()) , this , SLOT(zoom()));
        connect(&size , SIGNAL(modified()) , this , SLOT(refreshStatus()));
        if (size.exec() == QDialog::Accepted)
            cleanLists();
          else {
            unsaved = unsavedBackup;
            recoverBackup();
        }
      }
      break;
    case 205 :	//Depth conversion
      backup();
      {
	bool unsavedBackup = unsaved;
	unsaved = true;
	STiefenkonv tiefenkonv(this , &inhalt->bilddaten , undoList.last());
	connect(&tiefenkonv , SIGNAL(modified()) , this , SLOT(zoom()));
	connect(&tiefenkonv , SIGNAL(modified()) , this , SLOT(refreshStatus()));
	if (tiefenkonv.exec() == QDialog::Accepted)
	  cleanLists();
	else {
	  unsaved = unsavedBackup;
	  recoverBackup();
	}
      }
      break;
    case 206 :	//RGB adjust
      backup();
      emit dateiinfoChanged(tr("Not saved"));	//In cause of the fact that the statusbar will not change while executing the connection SRGBAnp::modified()<=>this::refreshStatus() is omittet in favour of this call
      {
	bool unsavedBackup = unsaved;
	unsaved = true;
	SRGBAnp RGBAnp(this , &inhalt->bilddaten , undoList.last());
	connect(&RGBAnp , SIGNAL(modified()) , this , SLOT(zoom()));
	if (RGBAnp.exec() == QDialog::Accepted)
	  cleanLists();
	else {
	  unsaved = unsavedBackup;
	  recoverBackup();
	}
      }
      break;
    case 207 :	//HSV adjust
      backup();
      emit dateiinfoChanged(tr("Not saved"));	//In cause of the fact that the statusbar will not change while executing the connection SFSHAnp::modified()<=>this::refreshStatus() is omittet in favour of this call
      {
	bool unsavedBackup = unsaved;
	unsaved = true;
	SFSHAnp FSHAnp(this , &inhalt->bilddaten , undoList.last());
	connect(&FSHAnp , SIGNAL(modified()) , this , SLOT(zoom()));
	if (FSHAnp.exec() == QDialog::Accepted)
	  cleanLists();
	else {
	  unsaved = unsavedBackup;
	  recoverBackup();
	}
      }
      break;
    case 208 :	//Gradation curve
      backup();
      emit dateiinfoChanged((tr("Not saved")));	//In cause of the fact that the statusbar will not change while executing the connection SGrad::modified()<=>this::refreshStatus() is omittet in favour of this call
      {
	bool unsavedBackup = unsaved;
	unsaved = true;
	SGrad grad(this , &inhalt->bilddaten , undoList.last());
	connect(&grad , SIGNAL(modified()) , this , SLOT(zoom()));
	if (grad.exec() == QDialog::Accepted)
	  cleanLists();
	else {
	  unsaved = unsavedBackup;
	  recoverBackup();
	}
      }
      break;
    case 209 :	//(Re-)Start macro recording
      macro.clear();
      recordingMacro = true;
      break;
    case 210 :	//Stop Recording
      recordingMacro = false;
      break;
    case 211 :	//Load macro
      {
        std::ifstream file(QFileDialog::getOpenFileName(QDir::homeDirPath() , "Yoltia Macro files (*.ym);;All files (*)" , this , "loadMacroDlg" , tr("Load macro from")));
        if (!file.fail()) {
            recordingMacro = false;
            macro.clear();
            int com;
            while (!file.eof()) {
              file >> com;
              macro.append(com);
            }
            file.close();
          } else
            emit meldung(tr("An error occured while trying to open file.") , SHORTDELAY);
      }
      break;
    case 212 :	//Save macro
      if (!macro.empty()) {
        QString filename = QFileDialog::getSaveFileName(QDir::homeDirPath() , "Yoltia Macro files (*.ym)" , this , "saveMacroDlg" , tr("Save macro as"));
        if (filename.lower().findRev(".ym") < int(filename.length() - 3))
          filename.append(".ym");
        using namespace std;
        ofstream file(filename);
        if (!file.fail()) {
            for (QValueList<int>::iterator it = macro.begin() ; it != macro.end() ; it++)
              file << setw(4) << *it;
            file.close();
          } else
            emit meldung(tr("An error occured while trying to open file.") , SHORTDELAY);
      }
      break;
    case 213 :  //Show actions
      if (!macro.empty()) {
        QStringList commandDict;
        commandDict.append(tr("Cut free marked region"));
        commandDict.append(tr("Invert"));
        commandDict.append(tr("Mirror vertically"));
        commandDict.append(tr("Mirror horizontally"));
        commandDict.append(tr("Fit to upper watched size"));
        commandDict.append(tr("Fit to lower watched size"));
        commandDict.append(tr("Remove 1 column of pixels from the right edge"));
        commandDict.append(tr("Remove 1 column of pixels from the left edge"));
        commandDict.append(tr("Remove 1 row of pixels from the lower edge"));
        commandDict.append(tr("Remove 1 row of pixels from the upper edge"));
        commandDict.append(tr("Remove 20 columns of pixels from the right edge"));
        commandDict.append(tr("Remove 20 columns of pixels from the left edge"));
        commandDict.append(tr("Remove 20 rows of pixels from the lower edge"));
        commandDict.append(tr("Remove 20 rows of pixels from the upper edge"));
        commandDict.append(tr("Remove 5 columns of pixels from the right edge"));
        commandDict.append(tr("Remove 5 columns of pixels from the left edge"));
        commandDict.append(tr("Remove 5 rows of pixels from the lower edge"));
        commandDict.append(tr("Remove 5 rows of pixels from the upper edge"));
        commandDict.append(tr("Auto-remove single colored margin around the picture"));
        QString text;
        for (QValueList<int>::iterator it = macro.begin() ; it != macro.end() ; it++)
          text.append(QString("%1\n").arg(commandDict[*it - 100]));
        QMessageBox::information(this , tr("Yoltia - Macro information") , tr("The macro contains the following actions:\n%1").arg(text) , QMessageBox::Ok , QMessageBox::NoButton , QMessageBox::NoButton);
      }
      break;
    case 214 :  //Save image as
      {
        QFileDialog speichern(QSettings().readEntry("/Yoltia/history/dir" , QDir::homeDirPath()) , "BMP (*.[bB][mM][pP]);;JPEG (*.[jJ][pP][gG] *.[jJ][pP][eE][gG]);;PBM (*.[pP][bB][mM]);;PGM (*.[pP][gG][mM]);;PNG (*.[pP][nN][gG]);;PPM (*.[pP][pP][mM]);;XBM (*.[xX][bB][mM]);;XPM (*.[xX][pP][mM])" , this , "speichern" , true);
        speichern.setSelectedFilter(1);
        speichern.setCaption(tr("Save image as"));
        speichern.setMode(QFileDialog::AnyFile);
        if (speichern.exec() == QDialog::Accepted and speichern.selectedFile()) {
          const QString filename = speichern.selectedFile();
          const QString endung = speichern.selectedFilter().left(speichern.selectedFilter().find(" "));	//Endbuchstaben aus dem Filter extrahieren
          if (filename.lower().findRev("." + endung.lower()) < int(filename.length() - endung.length() - 1))
              speichere(filename + "." + endung.lower() , endung);
            else
              speichere(filename , endung);
        }
      }
      break;
    case 215 :  //Print
      #ifndef QT_NO_PRINTER
      {
        QPrinter printer(QPrinter::PrinterResolution);
        if (printer.setup(this)) {
          QPainter p(&printer);
          if (p.isActive()) {
            if (inhalt->existsFixedMark())
                p.drawImage(0 , 0 , inhalt->bilddaten.copy(inhalt->markX , inhalt->markY , inhalt->markB , inhalt->markH));
              else
                p.drawImage(0 , 0 , inhalt->bilddaten);
          }
        }
      }
      #endif
      break;
  }
}

void SPicView::speichere(const QString &dateipfad , const QString &format) {
  actFormat = format;
  inhalt->bilddaten.save(dateipfad , format , QSettings().readNumEntry("/Yoltia/Quality/" + format , -1));	//Da nur JPEG & PNG g?ltig sind wird entweder die Komprimierung von diesen oder -1 zur?ckgegeben
  if (listMan.isOneMode())
    listMan.setDirectoryMode(dateipfad);
  if (dateipfad == listMan.act())        //Save or Save As
    unsaved = false;
  refreshStatus();
}

void SPicView::closeEvent(QCloseEvent *event) {
  if (notifyChanges())
      event->accept();
    else
      event->ignore();
}

void SPicView::refreshStatus() {
  {	// Creating geometry information and watching
    bool watchflag = false;
    QSettings einst;
    QString bildparm;
    int count = inhalt->bilddaten.width() * inhalt->bilddaten.height();
    if (einst.readBoolEntry("/Yoltia/Watch/WatchWidthMin" , false) and (inhalt->bilddaten.width() < einst.readNumEntry("/Yoltia/Watch/widthMinValue" , 100))) {
      watchflag = true;
      bildparm.append("!");
    }
    bildparm.append(QString::number(inhalt->bilddaten.width()));
    if (einst.readBoolEntry("/Yoltia/Watch/WatchWidthMax" , false) and (inhalt->bilddaten.width() > einst.readNumEntry("/Yoltia/Watch/widthMaxValue" , 100))) {
      watchflag = true;
      bildparm.append("!");
    }
    bildparm.append(215);
    if (einst.readBoolEntry("/Yoltia/Watch/WatchHeightMin" , false) and (inhalt->bilddaten.height() < einst.readNumEntry("/Yoltia/Watch/heightMinValue" , 100))) {
      watchflag = true;
      bildparm.append("!");
    }
    bildparm.append(QString::number(inhalt->bilddaten.height()));
    if (einst.readBoolEntry("/Yoltia/Watch/WatchHeightMax" , false) and (inhalt->bilddaten.height() > einst.readNumEntry("/Yoltia/Watch/heightMaxValue" , 100))) {
      watchflag = true;
      bildparm.append("!");
    }
    bildparm.append(char(215) + QString::number(inhalt->bilddaten.depth()) + " (" + QString::number(double(inhalt->bilddaten.height()) / double(inhalt->bilddaten.width())) + "), ");
    if (einst.readBoolEntry("/Yoltia/Watch/WatchCountMin" , false) and (count < einst.readNumEntry("/Yoltia/Watch/countMinValue" , 10000))) {
      watchflag = true;
      bildparm.append("!");
    }
    bildparm.append(QString::number(count));
    if (einst.readBoolEntry("/Yoltia/Watch/WatchCountMax" , false) and (count > einst.readNumEntry("/Yoltia/Watch/countMaxValue" , 10000))) {
      watchflag = true;
      bildparm.append("!");
    }
    inhalt->unsetPalette();
    if (watchflag) {
      if (einst.readBoolEntry("/Yoltia/Watch/beep" , false)) 
        QApplication::beep();
      if (einst.readBoolEntry("/Yoltia/Watch/changeColor" , false))
        inhalt->setPaletteBackgroundColor(QSettings().readEntry("/Yoltia/Watch/color" , "#ff0000"));
    }
    emit bildparamChanged(bildparm);
  }
  emit browseinfoChanged(listMan.browseInfo());
  if (inhalt->existsFixedMark())
      emit markChanged(QString::number(inhalt->markB) + char(215) + QString::number(inhalt->markH) + " + " + QString::number(inhalt->markX) + " + " + QString::number(inhalt->markY));
    else
      emit markChanged(0);      
  if (display) 
      emit zoomChanged(tr("%1: %2 %").arg(MODECHARS[zoomMode]).arg(qRound(inhalt->zoom * 100.0)));
    else
      emit zoomChanged(0);
  if (unsaved)
      emit dateiinfoChanged(tr("Not saved"));
    else
      emit dateiinfoChanged(QString::number(QFileInfo(listMan.act()).size() / 1024.0) + " kB @ " + actFormat);
  if (listMan.isOneMode())
      setCaption(tr("New picture"));
    else
      setCaption(listMan.act());
}

bool SPicView::notifyChanges() {
  bool ergebnis = true;
  if (unsaved)
    if (QImage::outputFormatList().grep(actFormat).isEmpty() or actFormat.isEmpty()) {	//Kann aktuelles Format nicht speichern?
	if (QMessageBox::information(this , tr("Yoltia - Close picture?") , tr("The current picture has got unsaved changes, that will be lost when closing.\nDo you want to close it anyway?") , QMessageBox::Yes , QMessageBox::No , QMessageBox::NoButton) == QMessageBox::No)
	  ergebnis = false;
      } else
	switch (QMessageBox::information(this , tr("Yoltia - Save changes?") , tr("The current picture has got unsaved changes, that will be lost when closing.\nDo you want to save the changes?") , QMessageBox::Yes , QMessageBox::No , QMessageBox::Abort)) {
	  case QMessageBox::Yes :
	    speichere(listMan.act() , actFormat);
	  case QMessageBox::No :
	    break;
	  default :
	    ergebnis = false;
	}
  return ergebnis;
}

void SPicView::cleanLists() {
  redoList.clear();
  const uchar undoN = QSettings().readNumEntry("/Yoltia/undoN" , 10);
  while (undoN < undoList.count())
    undoList.remove((uint) 0);
}

void SPicView::recoverBackup() {
  inhalt->bilddaten = undoList.last()->copy();
  undoList.remove();	//Delete last element
  zoom();
  refreshStatus();
}
